@@ -34,9 +34,9 @@
 #include <gtk/gtkbutton.h>
 #include <gtk/gtkhbox.h>
 #include <gtk/gtkmessagedialog.h>
+#include <gtk/gtkaboutdialog.h>
 #include <bonobo/bonobo-ui-component.h>
 #include <panel-applet-gconf.h>
-#include <libgnomeui/gnome-about.h>
 #include <libebook/e-book.h>
 #include <libebook/e-book-types.h>
 #include <libebook/e-contact.h>
@@ -97,57 +97,36 @@ entry_error_cb (EContactEntry *entry, const char* error) {
   gtk_widget_destroy (dialog);
 }
 
+static gboolean
+entry_button_press_cb (GtkWidget *widget, GdkEventButton *event, gpointer user_data)
+{
+  EvoLookupApplet *applet;
+  g_return_val_if_fail (user_data != NULL, FALSE);
+  applet = (EvoLookupApplet*)user_data;
+ 
+#ifndef USE_OLDER_PANEL_APPLET
+  panel_applet_request_focus (PANEL_APPLET (applet->applet_widget), event->time);
+#endif
+  // We just wanted to insert the above function call before the default action
+  return FALSE;
+}
+
 /**
  * About dialog callback.
  */
 static void
 about_cb (BonoboUIComponent *uic, gpointer user_data, const gchar *verbname)
 {
-  static GtkWidget *dialog = NULL;
   const char* authors[] = {"Ross Burton <ross@burtonini.com>", NULL};
 
-  if (dialog == NULL) {
-    dialog = gnome_about_new (PACKAGE_NAME, PACKAGE_VERSION,
-                              "Copyright \xc2\xa9 2003-2004 Ross Burton",
-                              _("An applet to search your address book."),
-                              authors, NULL, NULL, NULL);
-    g_signal_connect (dialog, "destroy", G_CALLBACK (gtk_widget_destroyed), &dialog);
-    /* TODO: just how evil is it to not cast to the correct gpointer*, just to stop the type-punning warning? */
-    g_object_add_weak_pointer (G_OBJECT (dialog), (gpointer)&dialog);
-    gtk_widget_show (dialog);
-  } else {
-    gtk_window_present (GTK_WINDOW (dialog));
-  }
-}
-
-static void
-parse_source_group(const char* string, ESourceGroup *completions)
-{
-  ESourceGroup *group;
-  GSList *l;
-
-  g_return_if_fail (string != NULL);
-  g_return_if_fail (completions != NULL);
-  
-  group = e_source_group_new_from_xml (string);
-  g_return_if_fail (group != NULL);
-  
-  for (l = e_source_group_peek_sources (group); l != NULL ; l = l->next) {
-    ESource *source;
-    const char *p;
-    source = E_SOURCE (l->data);
-    g_return_if_fail (source != NULL);
-    p = e_source_get_property (source, "completion");
-    if (p != NULL && strcmp (p, "true") == 0) {
-      ESource *new_source;
-      char *uri;
-      uri = g_strdup_printf("%s/%s", e_source_group_peek_base_uri (group), e_source_peek_relative_uri (source));
-      new_source = e_source_new_with_absolute_uri (e_source_peek_name (source), uri);
-      g_free (uri);
-      e_source_group_add_source (completions, new_source, 0);
-    }
-  }
-  g_object_unref (group);
+  gtk_show_about_dialog (NULL,
+			 "name", PACKAGE_NAME, 
+			 "version", PACKAGE_VERSION,
+			 "copyright", "Copyright \xc2\xa9 2003-2004 Ross Burton",
+			 "comments", _("An applet to search your address book."),
+			 "authors", authors,
+			 "logo-icon-name", "stock_contact",
+			 NULL);
 }
 
 static void
@@ -158,49 +137,28 @@ display_no_sources_dialog (void)
                                                GTK_BUTTONS_CLOSE,
                                                _("<span size='larger' weight='bold'>No Address Books Available</span>\n\n"
                                                  "There are no address books selected for auto-completion. "
-                                                 "To select which address books to use for completion, go to Tools -> Settings -> Autocompletion in Evolution."));
+                                                 "To select which address books to use for completion, go to Edit -> Preferences -> Autocompletion in Evolution."));
   /* TODO: better message */
   gtk_dialog_run (GTK_DIALOG (dialog));
   gtk_widget_destroy (dialog);
 }
 
-/**
- * GConf callback, the source key has changed. Free old book and load the new
- * one.
- */
 static void
-source_changed_cb (GConfClient* client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
+source_list_changed_cb (ESourceList *source_list, EvoLookupApplet *applet)
 {
-  EvoLookupApplet *applet = NULL;
-  GSList *list, *l;
-  ESourceGroup *sources = NULL;
-  g_return_if_fail (user_data != NULL);
-  g_return_if_fail (entry->value->type == GCONF_VALUE_LIST);
-  g_return_if_fail (gconf_value_get_list_type (entry->value) == GCONF_VALUE_STRING);
-
-  applet = (EvoLookupApplet*)user_data;
-  if (entry->value == NULL) {
-    /* Someone has unset this key, unref and disable the UI */
-    g_object_set (applet->entry_widget, "sources", NULL, NULL);
-    /* Now tell the user the applet won't work */
-    display_no_sources_dialog ();
-    return;
+  if (applet->source_list != NULL) {
+    g_object_unref (applet->source_list);
+    applet->source_list = NULL;
   }
 
-  list = gconf_value_get_list (gconf_entry_get_value (entry));
-  if (list != NULL) {
-    sources = e_source_group_new ("Completion", "");
-    for (l = list; l != NULL; l = l->next) {
-      GConfValue *value;
-      value = (GConfValue*)l->data;
-      parse_source_group (gconf_value_get_string (value), sources);
-    }
-  }
-  if (e_source_group_peek_sources (sources) == NULL) {
-    g_object_set (applet->entry_widget, "sources", NULL, NULL);  
-    display_no_sources_dialog ();
+  applet->source_list = e_source_list_new_for_gconf (applet->gconf_client, GCONF_COMPLETION_SOURCES);
+  e_contact_entry_set_source_list (E_CONTACT_ENTRY (applet->entry_widget),
+      applet->source_list);
+  if (applet->source_list != NULL && e_source_list_peek_groups (applet->source_list) != NULL) {
+    g_signal_connect (G_OBJECT (applet->source_list), "changed",
+	G_CALLBACK (source_list_changed_cb), applet);
   } else {
-    g_object_set (applet->entry_widget, "sources", sources, NULL);  
+    display_no_sources_dialog ();
   }
 }
 
@@ -213,14 +171,46 @@ change_size_cb (PanelApplet *widget, gint size, EvoLookupApplet *applet)
   theme_image (GTK_IMAGE (applet->icon), "stock_contact", size);
 }
 
+static void
+change_background_cb (PanelApplet *applet,
+		      PanelAppletBackgroundType type,
+		      GdkColor *colour,
+		      GdkPixmap *pixmap)
+{
+	GtkRcStyle *rc_style;
+	GtkStyle *style;
+
+	/* reset style */
+	gtk_widget_set_style (GTK_WIDGET (applet), NULL);
+	rc_style = gtk_rc_style_new ();
+	gtk_widget_modify_style (GTK_WIDGET (applet), rc_style);
+	g_object_unref (rc_style);
+
+	switch (type) {
+	case PANEL_NO_BACKGROUND:
+		break;
+	case PANEL_COLOR_BACKGROUND:
+		gtk_widget_modify_bg (GTK_WIDGET (applet),
+				      GTK_STATE_NORMAL, colour);
+		break;
+	case PANEL_PIXMAP_BACKGROUND:
+		style = gtk_style_copy (GTK_WIDGET (applet)->style);
+		if (style->bg_pixmap[GTK_STATE_NORMAL])
+			g_object_unref (style->bg_pixmap[GTK_STATE_NORMAL]);
+		style->bg_pixmap[GTK_STATE_NORMAL] = g_object_ref (pixmap);
+		gtk_widget_set_style (GTK_WIDGET (applet), style);
+		break;
+	}
+}
+
 /**
  * Callback from Bonobo when the control is destroyed.
  */
 static void
 bonobo_destroy_cb (BonoboObject *object, EvoLookupApplet *applet)
 {
-  gconf_client_notify_remove (applet->gconf_client, applet->source_notify_id);
   g_object_unref (applet->gconf_client);
+  g_object_unref (applet->source_list);
   g_free (applet);
 }
 
@@ -258,6 +248,7 @@ evo_lookup_applet_new (PanelApplet *parent_applet)
   gtk_entry_set_width_chars (GTK_ENTRY (applet->entry_widget), 15);
   g_signal_connect (applet->entry_widget, "contact-selected", G_CALLBACK (entry_contact_selected_cb), applet);
   g_signal_connect (applet->entry_widget, "error", G_CALLBACK (entry_error_cb), applet);
+  g_signal_connect (applet->entry_widget, "button_press_event", G_CALLBACK (entry_button_press_cb), applet);
   gtk_box_pack_start (GTK_BOX (hbox), applet->entry_widget, TRUE, TRUE, 0);
   
   gtk_widget_show_all (hbox);
@@ -275,15 +266,17 @@ evo_lookup_applet_new (PanelApplet *parent_applet)
   
   g_signal_connect (G_OBJECT (applet->applet_widget), "change_size",
                     G_CALLBACK (change_size_cb), applet);
+  g_signal_connect (G_OBJECT (applet->applet_widget), "change_background",
+		    G_CALLBACK (change_background_cb), applet);
   g_signal_connect (panel_applet_get_control (PANEL_APPLET (applet->applet_widget)), "destroy",
                     G_CALLBACK (bonobo_destroy_cb), applet);
   applet->gconf_client = gconf_client_get_default ();
   /* Monitor the Evolution completion preferences */
   gconf_client_add_dir (applet->gconf_client, GCONF_COMPLETION, GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
   /* Add a callback on source changes */
-  applet->source_notify_id = gconf_client_notify_add (applet->gconf_client, GCONF_COMPLETION_SOURCES, source_changed_cb, applet, NULL, NULL);
+  source_list_changed_cb (NULL, applet);
   /* TODO: if used in evo, monitor .../minimum_query_length and set as relevant on EContactEntry */
-  source_changed_cb (applet->gconf_client, -1, gconf_client_get_entry (applet->gconf_client, GCONF_COMPLETION_SOURCES, NULL, TRUE, NULL), applet);
+
   return TRUE;
 }
 
