@@ -25,8 +25,6 @@
  *
  *  Either a clear boolean property, or provide a _clear() method?
  *
- *  Allow developer to set additional/override query string (i.e. phonemgr).
- *
  *  set_sources() should keep a ref to the sources and connect signal handlers
  *  to manage the list. Or just expose a GList* of ESource*
  */
@@ -65,7 +63,7 @@ static int signals[LAST_SIGNAL] = { 0 };
 /* Properties */
 enum {
   PROP_0, /* TODO: why? */
-  PROP_SOURCES,
+  PROP_SOURCE_LIST,
   PROP_COMPLETE_LENGTH,
 };
 
@@ -73,15 +71,18 @@ enum {
  * TODO: store ESource* in EntryLookup, remove sources.
  */
 
-struct EContactEntry {
-  GtkEntry entry;
+struct EContactEntryPriv {
   GtkEntryCompletion *completion;
   GtkListStore *store;
-  ESourceGroup *sources;
+  ESourceList *source_list;
   /* A list of EntryLookup structs we are searching */
   GList *lookup_entries;
   /* Number of characters to start searching at */
   int lookup_length;
+  /* A null-terminated array of fields to complete on,
+   * n_search_fields includes the terminating NULL */
+  EContactField *search_fields;
+  int n_search_fields;
   /* Display callback */
   EContactEntryDisplayFunc display_func;
   gpointer display_data;
@@ -111,7 +112,7 @@ enum {
 G_DEFINE_TYPE(EContactEntry, e_contact_entry, GTK_TYPE_ENTRY);
 
 static void lookup_entry_free (EntryLookup *lookup);
-static EBookQuery* create_query (const char* s);
+static EBookQuery* create_query (EContactEntry *entry, const char* s);
 static guint entry_height (GtkWidget *widget);
 static const char* stringify_ebook_error(const EBookStatus status);
 
@@ -124,7 +125,7 @@ entry_activate_cb (EContactEntry *entry, gpointer user_data)
   GtkTreeIter iter;
 
   g_return_if_fail (E_IS_CONTACT_ENTRY (entry));
-  if (!gtk_tree_model_get_iter_first (GTK_TREE_MODEL (entry->store), &iter)) {
+  if (!gtk_tree_model_get_iter_first (GTK_TREE_MODEL (entry->priv->store), &iter)) {
     /*
      * Two possible situations:
      *
@@ -141,9 +142,11 @@ entry_activate_cb (EContactEntry *entry, gpointer user_data)
     EContact *contact;
     GError *error = NULL;
 
-    gtk_tree_model_get (GTK_TREE_MODEL (entry->store), &iter, COL_UID, &uid, COL_LOOKUP, &lookup, -1);
+    gtk_tree_model_get (GTK_TREE_MODEL (entry->priv->store), &iter, COL_UID, &uid, COL_LOOKUP, &lookup, -1);
     g_return_if_fail (lookup != NULL);
 
+    gtk_entry_set_text (GTK_ENTRY (entry), "");
+
     if (!e_book_get_contact (lookup->book, uid, &contact, &error)) {
       char* message;
       message = g_strdup_printf(_("Cannot get contact: %s"), error->message);
@@ -156,8 +159,7 @@ entry_activate_cb (EContactEntry *entry, gpointer user_data)
     }
     g_free (uid);
     
-    gtk_entry_set_text (GTK_ENTRY (entry), "");
-    gtk_list_store_clear (entry->store);
+    gtk_list_store_clear (entry->priv->store);
   }
 }
 
@@ -184,11 +186,11 @@ completion_match_selected_cb (GtkEntryCompletion *completion, GtkTreeModel *mode
     g_free (message);
     return FALSE;
   }
+  gtk_entry_set_text (GTK_ENTRY (entry), "");
   g_signal_emit (G_OBJECT (entry), signals[CONTACT_SELECTED], 0, contact);
   g_object_unref (contact);
   
-  gtk_entry_set_text (GTK_ENTRY (entry), "");
-  gtk_list_store_clear (entry->store);
+  gtk_list_store_clear (entry->priv->store);
   return TRUE;
 }
 
@@ -235,19 +237,25 @@ view_contacts_added_cb (EBook *book, GList *contacts, gpointer user_data)
 
     contact = E_CONTACT (contacts->data);
 
-    if (lookup->entry->display_func) {
-      string = lookup->entry->display_func (contact, lookup->entry->display_data);
+    if (lookup->entry->priv->display_func) {
+      string = lookup->entry->priv->display_func (contact, lookup->entry->priv->display_data);
     } else {
       string = g_strdup_printf ("%s <%s>", (char*)e_contact_get_const (contact, E_CONTACT_NAME_OR_ORG), (char*)e_contact_get_const (contact, E_CONTACT_EMAIL_1));
     }
 
+    /* Don't add the contact to the list if we don't have a string */
+    if (string == NULL) {
+      return;
+    }
+
     photo = e_contact_get (contact, E_CONTACT_PHOTO);
     if (photo) {
       GdkPixbufLoader *loader;
 
       loader = gdk_pixbuf_loader_new ();
 
-      if (gdk_pixbuf_loader_write (loader, photo->data, photo->length, NULL))
+      if (gdk_pixbuf_loader_write (loader, (guchar *)photo->data,
+			      photo->length, NULL))
         pixbuf = gdk_pixbuf_loader_get_pixbuf (loader);
 
       if (pixbuf) {
@@ -271,9 +279,9 @@ view_contacts_added_cb (EBook *book, GList *contacts, gpointer user_data)
       e_contact_photo_free (photo);
     }
     
-    gtk_list_store_append (lookup->entry->store, &iter);
+    gtk_list_store_append (lookup->entry->priv->store, &iter);
     /* At this point the matcher callback gets called */
-    gtk_list_store_set (lookup->entry->store, &iter,
+    gtk_list_store_set (lookup->entry->priv->store, &iter,
                         COL_NAME, string,
                         COL_UID, e_contact_get_const (contact, E_CONTACT_UID),
                         COL_PHOTO, pixbuf,
@@ -334,12 +342,12 @@ entry_changed_cb (GtkEditable *editable, gpointer user_data)
   EContactEntry *entry;
   entry = E_CONTACT_ENTRY (editable);
 
-  if (GTK_ENTRY (editable)->text_length >= entry->lookup_length) {
+  if (GTK_ENTRY (editable)->text_length >= entry->priv->lookup_length) {
     GList *l;
     EBookQuery *query;
 
     /* TODO: I appear to do this to stop duplicates, but its a bit of a hack */
-    for (l = entry->lookup_entries; l != NULL; l = l->next) {
+    for (l = entry->priv->lookup_entries; l != NULL; l = l->next) {
       EntryLookup *lookup;
       lookup = (EntryLookup*)l->data;
       if (lookup->bookview) {
@@ -348,10 +356,10 @@ entry_changed_cb (GtkEditable *editable, gpointer user_data)
       }
     }
     
-    gtk_list_store_clear (entry->store);
+    gtk_list_store_clear (entry->priv->store);
     
-    query = create_query (gtk_editable_get_chars (editable, 0, -1));
-    for (l = entry->lookup_entries; l != NULL; l = l->next) {
+    query = create_query (entry, gtk_editable_get_chars (editable, 0, -1));
+    for (l = entry->priv->lookup_entries; l != NULL; l = l->next) {
       EntryLookup *lookup;
       lookup = (EntryLookup*)l->data;
       if (e_book_async_get_book_view (lookup->book, query, NULL, 11, (EBookBookViewCallback)bookview_cb, lookup) != 0) {
@@ -388,61 +396,85 @@ book_opened_cb (EBook *book, EBookStatus status, gpointer data)
  */
 
 void
-e_contact_entry_set_sources (EContactEntry *entry, ESourceGroup *sources)
+e_contact_entry_set_source_list (EContactEntry *entry,
+    				  ESourceList *source_list)
 {
   GError *error = NULL;
-  GSList *s;
+  GSList *list, *l;
 
   g_return_if_fail (E_IS_CONTACT_ENTRY (entry));
 
-  /*
-   * TODO: define policy. Does this take a reference to the source, so calling
-   * code should unref; or does this own the copy passed to it?
-   */
-
   /* Release the old sources */  
-  if (entry->lookup_entries) {
-    g_list_foreach (entry->lookup_entries, (GFunc)lookup_entry_free, NULL);
-    g_list_free (entry->lookup_entries);
+  if (entry->priv->lookup_entries) {
+    g_list_foreach (entry->priv->lookup_entries, (GFunc)lookup_entry_free, NULL);
+    g_list_free (entry->priv->lookup_entries);
   }
-  if (entry->sources) {
-    g_object_unref (entry->sources);
+  if (entry->priv->source_list) {
+    g_object_unref (entry->priv->source_list);
   }
 
   /* If we have no new sources, disable and return here */
-  if (sources == NULL) {
+  if (source_list == NULL) {
     gtk_widget_set_sensitive (GTK_WIDGET (entry), FALSE);
-    entry->sources = NULL;
-    entry->lookup_entries = NULL;
+    entry->priv->source_list = NULL;
+    entry->priv->lookup_entries = NULL;
     return;
   }
 
-  entry->sources = sources;
+  entry->priv->source_list = source_list;
+  /* So that the list isn't going away underneath us */
+  g_object_ref (entry->priv->source_list);
 
-  entry->lookup_entries = NULL;
-  for (s = e_source_group_peek_sources (sources); s != NULL; s = s->next) {
-    EntryLookup *lookup;
-    lookup = g_new0 (EntryLookup, 1);
-    lookup->entry = entry;
+  /* That gets us a list of ESourceGroup */
+  list = e_source_list_peek_groups (source_list);
+  entry->priv->lookup_entries = NULL;
 
-    if ((lookup->book = e_book_new (s->data, &error)) == NULL) {
-      /* TODO handle this better, fire the error signal I guess */
-      g_warning (error->message);
-      g_error_free (error);
-      g_free (lookup);
-    } else {
-      entry->lookup_entries = g_list_append (entry->lookup_entries, lookup);
-      e_book_async_open(lookup->book, TRUE, (EBookCallback)book_opened_cb, lookup);
+  for (l = list; l != NULL; l = l->next) {
+    ESourceGroup *group = l->data;
+    GSList *sources = NULL, *m;
+    /* That should give us a list of ESource */
+    sources = e_source_group_peek_sources (group);
+    for (m = sources; m != NULL; m = m->next) {
+      ESource *source = m->data;
+      const char *p;
+
+      p = e_source_get_property (source, "completion");
+
+      if (p != NULL &&  strcmp (p, "true") == 0) {
+	ESource *s = e_source_copy (source);
+	EntryLookup *lookup;
+	char *uri;
+
+	uri = g_strdup_printf("%s/%s", e_source_group_peek_base_uri (group), e_source_peek_relative_uri (source));
+	e_source_set_absolute_uri (s, uri);
+	g_free (uri);
+
+	/* Now add those to the lookup entries list */
+	lookup = g_new0 (EntryLookup, 1);
+	lookup->entry = entry;
+
+	if ((lookup->book = e_book_new (s, &error)) == NULL) {
+	  /* TODO handle this better, fire the error signal I guess */
+	  g_warning (error->message);
+	  g_error_free (error);
+	  g_free (lookup);
+	} else {
+	  entry->priv->lookup_entries = g_list_append (entry->priv->lookup_entries, lookup);
+	  e_book_async_open(lookup->book, TRUE, (EBookCallback)book_opened_cb, lookup);
+	}
+
+	g_object_unref (s);
+      }
     }
   }
 }
 
-ESourceGroup *
-e_contact_entry_get_sources (EContactEntry *entry)
+ESourceList *
+e_contact_entry_get_source_list (EContactEntry *entry)
 {
   g_return_val_if_fail (E_IS_CONTACT_ENTRY (entry), NULL);
-  
-  return entry->sources;
+
+  return entry->priv->source_list;
 }
 
 void
@@ -451,8 +483,8 @@ e_contact_entry_set_complete_length (EContactEntry *entry, int length)
   g_return_if_fail (E_IS_CONTACT_ENTRY (entry));
   g_return_if_fail (length >= 1);
 
-  entry->lookup_length = length;
-  gtk_entry_completion_set_minimum_key_length (entry->completion, entry->lookup_length);
+  entry->priv->lookup_length = length;
+  gtk_entry_completion_set_minimum_key_length (entry->priv->completion, entry->priv->lookup_length);
 }
 
 int
@@ -460,7 +492,7 @@ e_contact_entry_get_complete_length (EContactEntry *entry)
 {
   g_return_val_if_fail (E_IS_CONTACT_ENTRY (entry), 3); /* TODO: from paramspec? */
   
-  return entry->lookup_length;
+  return entry->priv->lookup_length;
 }
 
 void
@@ -468,15 +500,30 @@ e_contact_entry_set_display_func (EContactEntry *entry, EContactEntryDisplayFunc
 {
   g_return_if_fail (E_IS_CONTACT_ENTRY (entry));
 
-  if (entry->display_destroy) {
-    entry->display_destroy (entry->display_func);
+  if (entry->priv->display_destroy) {
+    entry->priv->display_destroy (entry->priv->display_func);
   }
   
-  entry->display_func = func;
-  entry->display_data = func_data;
-  entry->display_destroy = destroy;
+  entry->priv->display_func = func;
+  entry->priv->display_data = func_data;
+  entry->priv->display_destroy = destroy;
 }
 
+void
+e_contact_entry_set_search_fields (EContactEntry *entry, const EContactField *fields)
+{
+  int i;
+
+  g_free (entry->priv->search_fields);
+  i = 0;
+  while (fields[i] != 0) {
+    i++;
+  }
+
+  entry->priv->search_fields = g_new0 (EContactField, i + 1);
+  memcpy (entry->priv->search_fields, fields, sizeof (EContactField) * (i + 1));
+  entry->priv->n_search_fields = i + 1;
+}
 
 /*
  *
@@ -493,8 +540,8 @@ e_contact_entry_set_property (GObject *object, guint property_id, const GValue *
   entry = E_CONTACT_ENTRY (object);
   
   switch (property_id) {
-  case PROP_SOURCES:
-    e_contact_entry_set_sources (entry, g_value_get_object (value));
+  case PROP_SOURCE_LIST:
+    e_contact_entry_set_source_list (entry, g_value_get_object (value));
     break;
   case PROP_COMPLETE_LENGTH:
     e_contact_entry_set_complete_length (entry, g_value_get_int (value));
@@ -512,8 +559,8 @@ e_contact_entry_get_property (GObject *object, guint property_id, GValue *value,
   entry = E_CONTACT_ENTRY (object);
   
   switch (property_id) {
-  case PROP_SOURCES:
-    g_value_set_object (value, e_contact_entry_get_sources (entry));
+  case PROP_SOURCE_LIST:
+    g_value_set_object (value, e_contact_entry_get_source_list (entry));
     break;
   case PROP_COMPLETE_LENGTH:
     g_value_set_int (value, e_contact_entry_get_complete_length (entry));
@@ -528,51 +575,74 @@ e_contact_entry_finalize (GObject *object)
 {
   GList *l;
   EContactEntry *entry = (EContactEntry *)object;
-  for (l = entry->lookup_entries; l != NULL; l = g_list_next (l)) {
-    lookup_entry_free (l->data);
-  }
-  g_list_free (entry->lookup_entries);
-  g_object_unref (entry->completion);
-  g_object_unref (entry->store);
-  g_object_unref (entry->sources);
-  if (entry->display_destroy) {
-    entry->display_destroy (entry->display_func);
+  if (entry->priv) {
+    for (l = entry->priv->lookup_entries; l != NULL; l = g_list_next (l)) {
+      lookup_entry_free (l->data);
+    }
+    g_free (entry->priv->search_fields);
+    g_list_free (entry->priv->lookup_entries);
+    g_object_unref (entry->priv->completion);
+    g_object_unref (entry->priv->store);
+    g_object_unref (entry->priv->source_list);
+
+    if (entry->priv->display_destroy) {
+      entry->priv->display_destroy (entry->priv->display_func);
+    }
+    g_free (entry->priv);
   }
   G_OBJECT_CLASS (e_contact_entry_parent_class)->finalize (object);
 }
 
+static void
+reset_search_fields (EContactEntry *entry)
+{
+  EContactField fields[] = { E_CONTACT_FULL_NAME, E_CONTACT_EMAIL, E_CONTACT_NICKNAME, E_CONTACT_ORG, 0 };
+
+  g_free (entry->priv->search_fields);
+  entry->priv->search_fields = g_new0 (EContactField, G_N_ELEMENTS (fields));
+  memcpy (entry->priv->search_fields, fields, sizeof (fields));
+  entry->priv->n_search_fields = G_N_ELEMENTS (fields);
+}
+
 static void
 e_contact_entry_init (EContactEntry *entry)
 {
   GtkCellRenderer *renderer;
 
+  entry->priv = g_new0 (EContactEntryPriv, 1);
   gtk_widget_set_sensitive (GTK_WIDGET (entry), FALSE);
 
   g_signal_connect (entry, "activate", G_CALLBACK (entry_activate_cb), NULL);
   g_signal_connect (entry, "changed", G_CALLBACK (entry_changed_cb), NULL);
 
-  entry->store = gtk_list_store_new (COL_TOTAL, G_TYPE_STRING, G_TYPE_STRING, GDK_TYPE_PIXBUF, G_TYPE_POINTER);
+  entry->priv->store = gtk_list_store_new (COL_TOTAL, G_TYPE_STRING, G_TYPE_STRING, GDK_TYPE_PIXBUF, G_TYPE_POINTER);
 
-  entry->completion = gtk_entry_completion_new ();
-  gtk_entry_completion_set_model (entry->completion, GTK_TREE_MODEL (entry->store));
-  gtk_entry_completion_set_match_func (entry->completion, (GtkEntryCompletionMatchFunc)completion_match_cb, NULL, NULL);
-  g_signal_connect (entry->completion, "match-selected", G_CALLBACK (completion_match_selected_cb), entry);
+  entry->priv->search_fields = NULL;
+  reset_search_fields (entry);
+
+  entry->priv->completion = gtk_entry_completion_new ();
+#if NEW_GTK
+  gtk_entry_completion_set_popup_set_width (entry->priv->completion, FALSE);
+#endif
+  gtk_entry_completion_set_model (entry->priv->completion, GTK_TREE_MODEL (entry->priv->store));
+  gtk_entry_completion_set_match_func (entry->priv->completion, (GtkEntryCompletionMatchFunc)completion_match_cb, NULL, NULL);
+  g_signal_connect (entry->priv->completion, "match-selected", G_CALLBACK (completion_match_selected_cb), entry);
   e_contact_entry_set_complete_length (entry, G_PARAM_SPEC_INT (g_object_class_find_property (G_OBJECT_GET_CLASS(entry), "complete-length"))->default_value);
-  gtk_entry_set_completion (GTK_ENTRY (entry), entry->completion);
+  gtk_entry_set_completion (GTK_ENTRY (entry), entry->priv->completion);
 
   /* Photo */
   renderer = gtk_cell_renderer_pixbuf_new ();
-  gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (entry->completion), renderer, TRUE);
-  gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (entry->completion), renderer, "pixbuf", COL_PHOTO);
+  gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (entry->priv->completion), renderer, TRUE);
+  gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (entry->priv->completion), renderer, "pixbuf", COL_PHOTO);
 
   /* Name */
   renderer = gtk_cell_renderer_text_new ();
-  gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (entry->completion), renderer, TRUE);
-  gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (entry->completion), renderer, "text", COL_NAME);
+  gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (entry->priv->completion), renderer, TRUE);
+  gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (entry->priv->completion), renderer, "text", COL_NAME);
 
-  entry->lookup_entries = NULL;
+  entry->priv->lookup_entries = NULL;
 
-  entry->display_func = entry->display_data = entry->display_destroy = NULL;
+  entry->priv->display_func = entry->priv->display_data = entry->priv->display_destroy = NULL;
 }
 
 static void
@@ -584,21 +654,15 @@ e_contact_entry_class_init (EContactEntryClass *klass)
   object_class = (GObjectClass *) klass;
   widget_class = (GtkWidgetClass *) klass;
   
-#if 0 /* TODO: required? */
-  /* GtkWidget */
-  widget_class->realize = bacon_cd_selection_realize;
-  widget_class->unrealize = bacon_cd_selection_unrealize;
-#endif
-
   /* GObject */
   object_class->set_property = e_contact_entry_set_property;
   object_class->get_property = e_contact_entry_get_property;
   object_class->finalize = e_contact_entry_finalize;
 
   /* Properties */
-  g_object_class_install_property (object_class, PROP_SOURCES,
-                                   g_param_spec_object ("sources", "Sources", "The sources to search for contacts.",
-                                                        E_TYPE_SOURCE_GROUP, G_PARAM_READWRITE));
+  g_object_class_install_property (object_class, PROP_SOURCE_LIST,
+                                   g_param_spec_object ("source-list", "Source List", "The source list to search for contacts.",
+                                                        E_TYPE_SOURCE_LIST, G_PARAM_READWRITE));
   
   g_object_class_install_property (object_class, PROP_COMPLETE_LENGTH,
                                    g_param_spec_int ("complete-length", "Complete length", "Number of characters to start a search on.",
@@ -688,28 +752,37 @@ split_query_string (const gchar *str)
  * nick name.
  */
 static EBookQuery*
-create_query (const char* s)
+create_query (EContactEntry *entry, const char* s)
 {
-  /* 
-   * TODO: Urgh. Lots of hard-coded integers here.
-   */
+  EBookQuery *query;
   GArray *parts = split_query_string (s);
   /* TODO: ORG doesn't appear to work */
-  gint fields[] = { E_CONTACT_FULL_NAME, E_CONTACT_EMAIL, E_CONTACT_NICKNAME, E_CONTACT_ORG };
-  EBookQuery **field_queries[4];
-  EBookQuery *q[4];
-  int i, j;
+  EBookQuery ***field_queries;
+  EBookQuery **q;
+  guint j;
+  int i;
 
-  for (i = 0; i < 4; i++) {
+  q = g_new0 (EBookQuery *, entry->priv->n_search_fields - 1);
+  field_queries = g_new0 (EBookQuery **, entry->priv->n_search_fields - 1);
+
+  for (i = 0; i < entry->priv->n_search_fields - 1; i++) {
     field_queries[i] = g_new0 (EBookQuery *, parts->len);
     for (j = 0; j < parts->len; j++) {
-        field_queries[i][j] = e_book_query_field_test (fields[i], E_BOOK_QUERY_CONTAINS, g_array_index (parts, gchar *, j));
+        field_queries[i][j] = e_book_query_field_test (entry->priv->search_fields[i], E_BOOK_QUERY_CONTAINS, g_array_index (parts, gchar *, j));
     }
     q[i] = e_book_query_and (parts->len, field_queries[i], TRUE);
   }
   g_array_free (parts, TRUE);
 
-  return e_book_query_or (4, q, TRUE);
+  query = e_book_query_or (entry->priv->n_search_fields - 1, q, TRUE);
+
+  for (i = 0; i < entry->priv->n_search_fields - 1; i++) {
+    g_free (field_queries[i]);
+  }
+  g_free (field_queries);
+  g_free (q);
+
+  return query;
 }
 
 /**
@@ -719,7 +792,7 @@ static guint
 entry_height (GtkWidget *widget)
 {
   PangoLayout *layout;
-  guint bound;
+  int bound;
   g_return_val_if_fail (widget != NULL, 0);
   layout = gtk_widget_create_pango_layout (widget, NULL);
   pango_layout_get_pixel_size (layout, NULL, &bound);
